CategoryButton = class();

local ImageFolder = "Simbo/NeoSindarin/Icons/Categories/";

local Constants =
{
	ButtonSize = 64,
	IconSize = 64,
	ButtonImages = 
	{
		Normal = ImageFolder .. "Blank.tga",
		Active = ImageFolder .. "Blank_selected.tga",
		Hover = ImageFolder .. "Blank_hover.tga"
	}
}

local IconOffset = (Constants.ButtonSize - Constants.IconSize) / 2;

local shortcutImages = {};

function CategoryButton:Constructor(parent, category, image, text, padding)
	self.Parent = parent;
	self.Category = category;
	self.Selected = false;
	self.Padding = padding or 0;
	
	self.Button = Turbine.UI.Button();
	self.Button:SetParent(parent);
	self.Button:SetSize(Constants.ButtonSize, Constants.ButtonSize);
	self.Button.Click = function(sender, args)
		self.Click(self, args);
	end
	self.Button.MouseEnter = function(sender, args)
		if not self.Selected then
			--self.ButtonImage:SetBackground(Constants.ButtonImages.Hover);
			SetIconImage(shortcutImages, self.ButtonImage, "Hover", Constants.ButtonImages);
		end
	end
	self.Button.MouseLeave = function(sender, args)
		if not self.Selected then
			--self.ButtonImage:SetBackground(Constants.ButtonImages.Normal);
			SetIconImage(shortcutImages, self.ButtonImage, "Normal", Constants.ButtonImages);
		end
	end
	
	self.ButtonImage = Turbine.UI.Control();
	self.ButtonImage:SetParent(parent);
	self.ButtonImage:SetSize(Constants.ButtonSize, Constants.ButtonSize);
	self.ButtonImage:SetZOrder(self.Button:GetZOrder() + 1);
	self.ButtonImage:SetMouseVisible(false);
	--self.ButtonImage:SetBackground(Constants.ButtonImages.Normal);
	SetIconImage(shortcutImages, self.ButtonImage, "Normal", Constants.ButtonImages);
	self.ButtonImage:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	
	self.ButtonIcon = Turbine.UI.Control();
	self.ButtonIcon:SetParent(parent);
	self.ButtonIcon:SetSize(Constants.IconSize, Constants.IconSize);
	self.ButtonIcon:SetZOrder(self.Button:GetZOrder() + 2);
	self.ButtonIcon:SetMouseVisible(false);
	self.ButtonIcon:SetBackground(ImageFolder .. image);
	self.ButtonIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);

	self.Label = Turbine.UI.Label();
	self.Label:SetParent(parent);
	self.Label:SetForeColor(Globals.Colours.CategoryButton);
	self.Label:SetFont(Globals.Fonts.CategoryButton);
	self.Label:SetSize(Constants.ButtonSize + self.Padding * 2, Globals.Sizes.CategoryButton);
	self.Label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.Label:SetText(text);

	self.Click = function() end;
end

function CategoryButton:SetSelected(selected)
	self.Selected = selected;
	if selected then
		--self.ButtonImage:SetBackground(Constants.ButtonImages.Active);
		SetIconImage(shortcutImages, self.ButtonImage, "Active", Constants.ButtonImages);
	else
		--self.ButtonImage:SetBackground(Constants.ButtonImages.Normal);
		SetIconImage(shortcutImages, self.ButtonImage, "Normal", Constants.ButtonImages);
	end
end

function CategoryButton:SetPosition(left, top)
	self.Button:SetPosition(left + self.Padding, top);
	self.ButtonImage:SetPosition(left + self.Padding, top);
	self.ButtonIcon:SetPosition(left + self.Padding + IconOffset, top + IconOffset);
	self.Label:SetPosition(left, top + Constants.ButtonSize);
end

function CategoryButton:GetHeight()
	return self.ButtonImage:GetHeight() + self.Label:GetHeight();
end

function CategoryButton:GetWidth()
	return self.ButtonImage:GetWidth();
end