function KeyActions(self, hideOnEscape)
	self:SetWantsKeyEvents(true);
	self.KeyDown = function(sender, args)
		if args.Action == 268435635 then
			if not self.UIHidden then
				self.UIHidden = true;
				self.WasVisible = self:IsVisible();
				if self.WasVisible then
					self:SetVisible(false);					
				end
			else
				if self.WasVisible then
					self:SetVisible(true);
				end
				self.UIHidden = false;
			end
		elseif args.Action == 145 and (self.AlwaysVisible == nil or not self.AlwaysVisible) then
			if self:IsVisible() then
				self.ReopenOnToggle = true;
			end
			self:SetVisible(false);
		end
	end
end

function CheckWindowPosition(window)
	local displayWidth, displayHeight = Turbine.UI.Display.GetSize();
	
	if window:GetLeft() + window:GetWidth() > displayWidth then
		window:SetLeft(displayWidth - window:GetWidth());
	end
	
	if window:GetLeft() < 0 then
		window:SetLeft(0);
	end
	
	if window:GetTop() + window:GetHeight() > displayHeight then
		window:SetTop(displayHeight - window:GetHeight());
	end
	
	if window:GetTop() < 0 then
		window:SetTop(0);
	end
end

function CreateButton(parent, wide)
	local button = Turbine.UI.Lotro.Button();
	button:SetParent(parent);
	if wide then
		button:SetSize(Globals.Sizes.WideButton.Width, Globals.Sizes.WideButton.Height);
	else
		button:SetSize(Globals.Sizes.Button.Width, Globals.Sizes.Button.Height);
	end
	button:SetText("");
	
	return button;
end

function CreateTextBox(parent, multiline, bordered)
	if multiline == nil then
		multiline = false;
	end
	
	local textBox;
	if bordered ~= nil then
		textBox = BorderedTextBox();
		textBox:SetBorderColour(Globals.BorderColours.TextBox);
	else
		textBox = Turbine.UI.Lotro.TextBox();
	end
	textBox:SetParent(parent);
	textBox:SetMultiline(multiline);
	textBox:SetFont(Globals.Fonts.TextBox);
	if not multiline then
		textBox:SetSize(Globals.Sizes.TextBox.Width, Globals.Sizes.TextBox.Height);
	end
	textBox:SetForeColor(Globals.Colours.TextBox);
	textBox:SetBackColor(Globals.BackgroundColours.TextBox);
	textBox:SetText("");

	return textBox;
end

function CreateLabel(parent, mouseVisible, left, top)
	if mouseVisible == nil then
		mouseVisible = false;
	end
	
	local label = Turbine.UI.Label();
	label:SetParent(parent);
	label:SetForeColor(Globals.Colours.Label);
	label:SetFont(Globals.Fonts.Label);
	label:SetSize(Globals.Sizes.Label.Width, Globals.Sizes.Label.Height);
	label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	label:SetMouseVisible(mouseVisible);
	label:SetText("");
	
	if left ~= nil and top ~= nil then
		label:SetPosition(left, top);
	end

	return label;
end

function CreateRadioButton(parent, text, groupName, left, top, width, height, image1, image2)
	local radioButton = RadioButton(parent);
	if left ~= nil and top ~= nil then
		radioButton:SetPosition(left, top);
	end
	if width ~= nil and height ~= nil then
		radioButton:SetSize(width, height);
	end
	radioButton:SetForeColor(Globals.Colours.Label);
	radioButton:SetFont(Globals.Fonts.Label);
	radioButton:SetText(text);
	radioButton:SetGroup(groupName);
	
	if image1 ~= nil then
		CreateRadioButtonImage(radioButton, image1, left + width, top, width, height);
	end	
	
	if image2 ~= nil then
		CreateRadioButtonImage(radioButton, image2, left + width * 2, top, width, height);
	end
	
	return radioButton;
end

function CreateRadioButtonImage(radioButton, imageName, left, top, width, height)
	local image = Turbine.UI.Control();
	image:SetParent(radioButton.Parent);
	image:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	image:SetBackground(imageName);
	image:SetPosition(left, top);
	image:SetSize(width, height);
	image.MouseClick = function(sender, args)
		radioButton.MouseClick(sender, args);
	end
end

function CreateDropdown(parent, left, top, width, height, dropdownHeight)
	local dropdown = DropdownList(parent);
	
	dropdown:SetBorderColor(Globals.BorderColours.ListBox);
	dropdown:SetBackColor(Globals.BackgroundColours.ListBox);	
	dropdown:SetForeColor(Globals.Colours.ListBox);	
	dropdown:SetFont(Globals.Fonts.ListBox);
	if width ~= nil and height ~= nil then
		dropdown:SetSize(width, height, dropdownHeight);
	end
	if left ~= nil and top ~= nil then
		dropdown:SetPosition(left, top);
	end

	return dropdown;
end

function CreateCheckBox(parent, text, left, top, width, height)
	local checkBox = Turbine.UI.Lotro.CheckBox();
	checkBox:SetParent(parent);
	if left ~= nil and top ~= nil then
		checkBox:SetPosition(left, top);
	end
	if width ~= nil and height ~= nil then
		checkBox:SetSize(width, height);
	end
	checkBox:SetForeColor(Globals.Colours.Label);
	checkBox:SetFont(Globals.Fonts.Label);
	checkBox:SetText(" " .. text);
	checkBox:SetChecked(false);
	
	return checkBox;
end

function SetIconImage(shortcutImages, icon, imageType, iconSet)
	iconSet = iconSet or Globals.Icons;
	
	if shortcutImages[imageType] == nil then
		icon:SetBackground(iconSet[imageType]);
		shortcutImages[imageType] = icon:GetBackground();
	else
		icon:SetBackground(shortcutImages[imageType]);
	end
end