import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Simbo.NeoSindarin.Class"; -- Turbine library included so that there's no outside dependencies
import "Simbo.NeoSindarin.Localisation";
import "Simbo.NeoSindarin.Functions_Windows";
import "Simbo.NeoSindarin.StringFunctions";
import "Simbo.NeoSindarin.ShortcutButton";
import "Simbo.NeoSindarin.CategoryButton";
import "Simbo.NeoSindarin.DropdownList";
import "Simbo.NeoSindarin.Dictionary";
import "Simbo.NeoSindarin";

Settings = 
{
	NeoSindarinWindow =
	{
		Position = {Left = "100", Top = "100"},
		Size = {Width = "400", Height = "350"},
		WindowOpacity = "96",
		DefaultChannel = Strings["cmd_say"],
		HideTranslation = false
	},
	ToggleWindow =
	{
		Position = {Left = "100", Top = "100"},
		WindowOpacity = "96"
	}
};

Globals =
{
	Sizes =
	{
		Label = { Width = 130, Height = 22 },
		TextBox = { Width = 100, Height = 22 },
		Button = { Width = 90, Height = 24 },
		CheckBox = { Width = 200, Height = 22 },
		WideButton = { Width = 120, Height = 24 },
		DropdownList = { Width = 100, Height = 24, OpenedHeight = 120 },
		ShortcutButton = { Width = 100,	Height = 18 },
		CategoryButton = 12
	},
	Fonts = 
	{
		Label = Turbine.UI.Lotro.Font.TrajanPro15,
		CategoryButton = Turbine.UI.Lotro.Font.TrajanPro14,
		TextBox = Turbine.UI.Lotro.Font.TrajanPro15,
		ListBox = Turbine.UI.Lotro.Font.TrajanPro14
	},
	Colours =
	{
		Label = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		CategoryButton = Turbine.UI.Color(1.0, 0.85, 0.85, 0.62),
		TextBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		ListBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		Disabled = Turbine.UI.Color(1.0, 0.33, 0.33, 0.33)
	},
	BackgroundColours =
	{
		TextBox = Turbine.UI.Color(1.0, 0, 0, 0),
		ListBox = Turbine.UI.Color(1.0, 0, 0, 0),
		RadioButton = Turbine.UI.Color(1.0, 0, 0, 0)
	},
	BorderColours =
	{
		TextBox = Turbine.UI.Color(1.0, 0.10, 0.23, 0.07),
		ListBox = Turbine.UI.Color(1.0, 0.10, 0.23, 0.07)
	},
	Icons = 
	{
		Toggle = "Simbo/NeoSindarin/Icons/Toggle.tga",
		Toggle_Hover = "Simbo/NeoSindarin/Icons/Toggle_Hover.tga",
		WindowResizer = "Simbo/NeoSindarin/Icons/MouseDrag.tga",
		MainShortcut_Normal = "Simbo/NeoSindarin/Icons/MainShortcut.tga",
		MainShortcut_Hover = "Simbo/NeoSindarin/Icons/MainShortcut_Hover.tga",
		MainShortcut_Inactive = "Simbo/NeoSindarin/Icons/MainShortcut_Inactive.tga",
		SortedAsc = "Simbo/NeoSindarin/Icons/SortedAsc.tga",
		SortedAsc_Hover = "Simbo/NeoSindarin/Icons/SortedAsc_Hover.tga",
		SortedDesc = "Simbo/NeoSindarin/Icons/SortedDesc.tga",
		SortedDesc_Hover = "Simbo/NeoSindarin/Icons/SortedDesc_Hover.tga",
		SortedNone = "Simbo/NeoSindarin/Icons/SortedNone.tga",
		SortedNone_Hover = "Simbo/NeoSindarin/Icons/SortedNone_Hover.tga",
		ShortcutButton = 
		{
			Normal = "Simbo/NeoSindarin/Icons/BlankButton.tga",
			Hover = "Simbo/NeoSindarin/Icons/BlankButtonHover.tga",
			Inactive = "Simbo/NeoSindarin/Icons/BlankButtonInactive.tga"
		}
	}
}

NeoSindarinLoad = Turbine.PluginData.Load;
NeoSindarinSave = Turbine.PluginData.Save;

Settings = NeoSindarinLoad(Turbine.DataScope.Account, "NeoSindarinSettings") or Settings;

Settings.ToggleWindow = Settings.ToggleWindow or { Position = {Left = "100", Top = "100"}, WindowOpacity = "96" };

if Settings.NeoSindarinWindow.Opacity ~= nil then
	Settings.NeoSindarinWindow.WindowOpacity = "96";
	Settings.NeoSindarinWindow.Opacity = nil;
end

if Settings.ToggleWindow.Opacity ~= nil then
	Settings.ToggleWindow.WindowOpacity = "96";
	Settings.ToggleWindow.Opacity = nil;
end

neoSindarinWindow = Simbo.NeoSindarin.NeoSindarinWindow();
if Settings.NeoSindarinWindow.Visible == nil or Settings.NeoSindarinWindow.Visible == "yes" then
	neoSindarinWindow:SetVisible(true);
else
	neoSindarinWindow:SetVisible(false);
end

toggleWindow = Simbo.NeoSindarin.ToggleWindow();
toggleWindow:SetVisible(true);
toggleWindow:SetZOrder(neoSindarinWindow:GetZOrder() + 1);
CheckWindowPosition(toggleWindow);

if (Plugins["Neo-Sindarin"] ~= nil ) then
	Plugins["Neo-Sindarin"].Unload = function( sender, args )
		neoSindarinWindow:SaveSettings(true);
	end
end

neoSindarinCommand = Turbine.ShellCommand();

function neoSindarinCommand:Execute(cmd, args)
	args = string.lower(args);

	if args == Strings["cmd_show"] then
		neoSindarinWindow:Show();
	elseif args == Strings["cmd_hide"] then
		neoSindarinWindow:Hide();
	elseif args == Strings["cmd_toggle"] then
		neoSindarinWindow:Toggle();
	elseif args ~= nil then
		neoSindarinCommand:GetHelp();
	end
end

function neoSindarinCommand:GetHelp()
	Turbine.Shell.WriteLine(Strings["usage"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help1"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help2"]);
	Turbine.Shell.WriteLine(" " .. Strings["cmd_help3"]);
end

Turbine.Shell.AddCommand(Strings["cmd_neosindarin"], neoSindarinCommand);
Turbine.Shell.WriteLine("Neo-Sindarin v" .. Plugins["Neo-Sindarin"]:GetVersion() .. " by Simbo");

local text = "___B_1___C___";
local test = text:Split("___", false);