PhrasesListBox = class();

local Constants =
{
	Border = 1,
	ScrollBarSize = 10,
	LineSize = 20,
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25)
	}
}

function PhrasesListBox:Constructor(parent)
	self.Parent = parent;
	self.LineCount = 0;
	self.SelectedIndex = -1;
	self.Lines = {};
	
	self.Container = Turbine.UI.Control();
	self.Container:SetParent(parent);
	self.Container:SetBackColor(Globals.BorderColours.ListBox);

	self.LinesBox = Turbine.UI.ListBox();
	self.LinesBox:SetParent(self.Container);
	self.LinesBox:SetPosition(Constants.Border, Constants.Border)
	self.LinesBox:SetBackColor(Globals.BackgroundColours.ListBox);
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetParent(self.Container);
	self.LinesScrollBar:SetWidth(Constants.ScrollBarSize)
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesBox:SetVerticalScrollBar(self.LinesScrollBar);

	self.LinesBox.SelectedIndexChanged = function(sender, args)
		if self.SelectedIndex > 0 and self.SelectedIndex <= self.LinesBox:GetItemCount() then
			local line = self.LinesBox:GetItem(self.SelectedIndex);
			line:SetBackColor(Globals.BackgroundColours.ListBox);
		end
		
		self.SelectedIndex = self.LinesBox:GetSelectedIndex();
		
		local index = self.SelectedIndex;
		if index > 0 then
			local line = self.LinesBox:GetItem(index);
			line:SetBackColor(Constants.Colours.SelectedItem);
			self.SelectedIndexChanged(self, {
				Index = index,
				Data = line.Data,
				Text = line:GetText()
			});
		else
			self.SelectedIndexChanged(self, {
				Index = index,
				Data = nil,
				Text = ""
			});
		end
	end
	
	self.SelectedIndexChanged = function() end;
end

function PhrasesListBox:GetLeft()
	return self.Container:GetLeft();
end

function PhrasesListBox:GetTop()
	return self.Container:GetTop();
end

function PhrasesListBox:GetWidth()
	return self.Container:GetWidth();
end

function PhrasesListBox:GetHeight()
	return self.Container:GetHeight();
end

function PhrasesListBox:SetPosition(left, top)
	self.Container:SetPosition(left, top);
end

function PhrasesListBox:SetSize(width, height)
	local linesBoxWidth = width - Constants.Border * 2;
	local linesBoxHeight = height - Constants.Border * 2;
	
	self.Container:SetSize(width, height);
	self.LinesBox:SetSize(linesBoxWidth, linesBoxHeight);

	self.LinesScrollBar:SetPosition(self.LinesBox:GetLeft() + linesBoxWidth - Constants.ScrollBarSize, self.LinesBox:GetTop());
	self.LinesScrollBar:SetHeight(linesBoxHeight);
	
	local itemCount = self.LinesBox:GetItemCount();
	for i = 1, itemCount do
		self.LinesBox:GetItem(i):SetWidth(linesBoxWidth);
	end
end

function PhrasesListBox:AddItem(text, data, disabledFlag)
	local line = nil;
	
	if self.LineCount < table.getn(self.Lines) then
		line = self.Lines[self.LineCount + 1];
	else
		line = self:CreateLine();
		table.insert(self.Lines, self.LineCount + 1, line);
	end
	
	line.DisabledFlag = disabledFlag;
	line:SetText(text);
	line.Data = data;

	self.LinesBox:AddItem(line);
	self.LineCount = self.LineCount + 1;
end

function PhrasesListBox:ShowDisabledItems(disabledFlag)
	local itemCount = self.LinesBox:GetItemCount();
	for i = 1, itemCount do
		local line = self.LinesBox:GetItem(i);
		if line.DisabledFlag == disabledFlag then
			line:SetForeColor(Globals.Colours.Disabled);
		else
			line:SetForeColor(Globals.Colours.Label);
		end
	end
end

function PhrasesListBox:ClearItems()
	self.LinesBox:SetSelectedIndex(0);
	self.LinesBox:ClearItems();
end

function PhrasesListBox:CreateLine()
	local line = CreateLabel(self.LinesBox, true);
	line:SetSize(self.LinesBox:GetWidth(), Constants.LineSize);
	line:SetMultiline(false);
	line:SetBackColor(Globals.BackgroundColours.ListBox);
		
	line.MouseEnter = function(sender, args)
		line:SetOutlineColor(Constants.Colours.HoverOutline);
		line:SetFontStyle(Turbine.UI.FontStyle.Outline);
	end
	line.MouseLeave = function(sender, args)
		line:SetOutlineColor(Constants.Colours.NormalOutline);
		line:SetFontStyle(Turbine.UI.FontStyle.None);
	end
	
	return line;
end

function PhrasesListBox:SelectLine(index)
	self.LinesBox:SetSelectedIndex(index);
end