import "Simbo.NeoSindarin.PhrasesListBox";

NeoSindarinWindow = class(WindowLotro);

local Constants =
{
	Padding =
	{
		Top = 40,
		Left = 10,
		Right = 10,
		Bottom = 20,
		Lines = 6,
		Columns = 6
	},
	Sizes =
	{
		Categories = 180,
		PhraseBox = 48,
		ReadButton = 32,
		TextBox = 200,
		CheckBox = 200,
		SortLabel = 75,
		Dropdown = 
		{
			Width = 140,
			Height = 20,
			Open = 135,
			OpenDouble = 40
		},
		SortIcon =
		{
			Width = 15,
			Height = 15
		}
	},
	MinimumSize =
	{
		Width = UIOptions["minimum_width"];
		Height = 420
	}
}

local shortcutImages = {};

function NeoSindarinWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.SelectedCategory = nil;
	self.SelectedDialect = (Settings.NeoSindarinWindow.DefaultDialect or "Exilic");
	self.SelectedFormality = (Settings.NeoSindarinWindow.DefaultFormality or "Formal");
	self.SelectedPhrase = nil;
	self.SelectedPhraseText = "";
	self.MoreInfoIsShown = false;
	self.SelectedButton = nil;
	self.DisplayedPhrases = {};
	self.SortOrder = nil;
	
	self:SetText(Strings["neosindarin"]);
	self:SetPosition(tonumber(Settings.NeoSindarinWindow.Position.Left), tonumber(Settings.NeoSindarinWindow.Position.Top));
	
	self:SetSize(tonumber(Settings.NeoSindarinWindow.Size.Width), tonumber(Settings.NeoSindarinWindow.Size.Height));
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self:SetOpacity(tonumber(Settings.NeoSindarinWindow.WindowOpacity) / 100);
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	local height = 0;
	
	local padding = UIOptions["button_padding"];
	left = left - padding;
	
	for index, categoryData in ipairs(NeoSindarinDB.Categories) do 
		local button = CategoryButton(self, categoryData.Key, categoryData.Image, categoryData[LANGUAGE] or categoryData["EN"], padding);
		button:SetPosition(left, top);
		button.Click = function(sender, args)
			if self.SelectedCategory ~= sender.Category then
				if self.SelectedButton ~= nil then
					self.SelectedButton:SetSelected(false)
					self.SelectedButton = sender;
				end
				self.SelectedButton = sender;
				
				self.SelectedCategory = sender.Category;
				self.SelectedButton:SetSelected(true);
				self:PopulatePhrases();
			end
		end
		left = left + button:GetWidth() + (padding * 2);
		height = button:GetHeight();
	end
	
	top = top + height + Constants.Padding.Lines * 2;
	left = Constants.Padding.Left;
	
	local dialectLabel = CreateLabel(self, false, left, top);
	dialectLabel:SetText(Strings["dialect"]);
	
	left = left + dialectLabel:GetWidth();
	
	self.DialectList = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.OpenDouble);
	self.DialectList:AddItem(Strings["dialect_exilic"], "Exilic");
	self.DialectList:AddItem(Strings["dialect_woodelven"], "Woodelven");
	self.DialectList:SetSelectedValue(self.SelectedDialect, false);
	self.DialectList.SelectedIndexChanged = function(sender, args)
		self.SelectedDialect = args.SelectedValue;
		self.PhrasesListBox:ShowDisabledItems(args.SelectedValue);
		self:ShowTranslatedText(true);
		self.DialectInfoLabel:SetText(Strings["dialect_info_" .. self.SelectedDialect]);
	end
	
	left = left + Constants.Sizes.Dropdown.Width + Constants.Padding.Columns;
	
	self.FormalityList = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.OpenDouble);
	self.FormalityList:AddItem(Strings["formal"], "Formal");
	self.FormalityList:AddItem(Strings["informal"], "Informal");
	self.FormalityList:SetSelectedValue(self.SelectedFormality, false);
	self.FormalityList.SelectedIndexChanged = function(sender, args)
		self.SelectedFormality = args.SelectedValue;
		self:ShowTranslatedText(true);
	end
	
	top = top + dialectLabel:GetHeight();
	left = self.DialectList:GetLeft();
		
	self.DialectInfoLabel = CreateLabel(self, false, left, top);
	self.DialectInfoLabel:SetText(Strings["dialect_info_" .. self.SelectedDialect]);
	
	top = top + self.DialectInfoLabel:GetHeight() + Constants.Padding.Lines;
	left = Constants.Padding.Left;
		
	local channelListLabel = CreateLabel(self, false, left, top);
	channelListLabel:SetText(Strings["output_channel"]);
	
	left = left + channelListLabel:GetWidth();
	
	self.ChannelList = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.ChannelList:AddItem(Strings["say"], Strings["cmd_say"]);
	self.ChannelList:AddItem(Strings["fellowship"], Strings["cmd_fellowship"]);
	self.ChannelList:AddItem(Strings["kinship"], Strings["cmd_kinship"]);
	self.ChannelList:AddItem(Strings["raid"], Strings["cmd_raid"]);
	self.ChannelList:AddItem(Strings["userchat1"], Strings["cmd_userchat1"]);
	self.ChannelList:AddItem(Strings["userchat2"], Strings["cmd_userchat2"]);
	self.ChannelList:AddItem(Strings["userchat3"], Strings["cmd_userchat3"]);
	self.ChannelList:AddItem(Strings["userchat4"], Strings["cmd_userchat4"]);
	self.ChannelList:SetSelectedValue(Settings.NeoSindarinWindow.DefaultChannel or Strings["cmd_say"], false);
	self.ChannelList.SelectedIndexChanged = function(sender, args)
		self:ShowTranslatedText(true);
	end
	
	top = top + Constants.Sizes.Dropdown.Height + Constants.Padding.Lines;
	left = Constants.Padding.Left;
	
	local filterLabel = CreateLabel(self, false, left, top);
	filterLabel:SetText(Strings["filter"]);
	
	left = left + channelListLabel:GetWidth();
		
	self.FilterBox = CreateTextBox(self, false, true);
	self.FilterBox:SetPosition(left, top);
	self.FilterBox:SetWidth(Constants.Sizes.TextBox);
	self.FilterBox.FocusGained = function (sender, args)
		self.FilterBox.SaveText = self.FilterBox:GetText();
		self.FilterBox:SetWantsUpdates(true);
	end
	self.FilterBox.FocusLost = function (sender, args)
		self.FilterBox:SetWantsUpdates(false);
	end
	self.FilterBox.Update = function (sender, args)
		local text = sender:GetText();
		if text ~= sender.SaveText then
			sender.SaveText = text;
			self:FilterPhrases(text);
		end
	end
	
	top = top + filterLabel:GetHeight() + Constants.Padding.Lines;
	left = Constants.Padding.Left;
	
	self.PhrasesListBox = PhrasesListBox(self);
	self.PhrasesListBox:SetPosition(left, top);
	self.PhrasesListBox.SelectedIndexChanged = function(sender, args)
		if args.Index > 0 then
			self.SelectedPhrase = args.Data;
			self.SelectedPhraseText = args.Text;
			
			local info = args.Data.Info;
			if info ~= nil then
				self.InfoBox:SetText(info[LANGUAGE] or info["EN"]);
			else
				self.InfoBox:SetText("");
			end
		else
			self.SelectedPhrase = nil;
			self.SelectedPhraseText = "";
			self.InfoBox:SetText("");
		end
		self:ShowTranslatedText(not self.FilterBox:GetWantsUpdates());
	end
	
	self.SortIcon = Turbine.UI.Control();
	self.SortIcon:SetParent(self);
	self.SortIcon:SetSize(Constants.Sizes.SortIcon.Width, Constants.Sizes.SortIcon.Height);
	self.SortIcon:SetTop(top - Constants.Sizes.SortIcon.Height - Constants.Padding.Lines);
	self.SortIcon.SetIcon = function(hover)
		local icon;
		
		if self.SortOrder == "Asc" then
			if hover then
				icon = "SortedAsc_Hover";
			else
				icon = "SortedAsc";
			end
		elseif self.SortOrder == "Desc" then
			if hover then
				icon = "SortedDesc_Hover";
			else
				icon = "SortedDesc";
			end
		else
			if hover then
				icon = "SortedNone_Hover";
			else
				icon = "SortedNone";
			end
		end
		--self.SortIcon:SetBackground(icon);
		SetIconImage(shortcutImages, self.SortIcon, icon);
	end
	self.SortIcon.SetIcon(false);
	self.SortIcon.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			if self.SortOrder == "Asc" then
				self.SortOrder = "Desc"
				self:SortPhrases();
			elseif self.SortOrder == "Desc" then
				self.SortOrder = nil;
				self:PopulatePhrases();
			else
				self.SortOrder = "Asc";
				self:SortPhrases();
			end
			self.SortIcon.SetIcon(true);
		end
	end	
	self.SortIcon.MouseEnter = function(sender,args)
		self.SortIcon.SetIcon(true);
	end
	self.SortIcon.MouseLeave = function(sender,args)
		self.SortIcon.SetIcon(false);
	end
	
	self.SortLabel = CreateLabel(self, false, 0, self.SortIcon:GetTop());
	self.SortLabel:SetSize(Constants.Sizes.SortLabel, self.SortIcon:GetHeight());
	self.SortLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.SortLabel:SetText(Strings["sort"]);
	
	self.InfoBox = CreateTextBox(self, true, true);
	self.InfoBox:SetPosition(left, top);
	self.InfoBox:SetHeight(Constants.Sizes.PhraseBox);
	self.InfoBox:SetEnabled(false);
	self.InfoBox:SetBackColor(Globals.Colours.Disabled);
	self.InfoBox:SetVisible(false);
	
	--[[
	self.MoreInfoButton = CreateButton(self, 0);
	self.MoreInfoButton:SetText(Strings["show_info"]);
	self.MoreInfoButton.MouseClick = function(sender, args)
		if self.MoreInfoIsShown then
			self.MoreInfoButton:SetText(Strings["show_info"]);
		else
			self.MoreInfoButton:SetText(Strings["hide_info"]);
		end
		self.MoreInfoIsShown = not self.MoreInfoIsShown;
		self.InfoBox:SetVisible(self.MoreInfoIsShown);
		self:LayoutControls();
	end
	]]
	self.MoreInfoButton = ShortcutButton(true);
	self.MoreInfoButton:SetParent(self);
	self.MoreInfoButton:SetPosition(left, top);
	self.MoreInfoButton:SetSize(Globals.Sizes.ShortcutButton.Width, Globals.Sizes.ShortcutButton.Height);
	self.MoreInfoButton:SetIcons(Globals.Icons.ShortcutButton);
	self.MoreInfoButton:SetText(Strings["show_info"]);
	self.MoreInfoButton.QuickslotClicked = function(sender, args)
		if self.MoreInfoIsShown then
			self.MoreInfoButton:SetText(Strings["show_info"]);
		else
			self.MoreInfoButton:SetText(Strings["hide_info"]);
		end
		self.MoreInfoIsShown = not self.MoreInfoIsShown;
		self.InfoBox:SetVisible(self.MoreInfoIsShown);
		self:LayoutControls();
	end
	
	self.PlaceHolderLabel = CreateLabel(self, false, left, top);
	self.PlaceHolderLabel:SetText(Strings["placeholder_text"]);
	self.PlaceHolderLabel:SetForeColor(Globals.Colours.Disabled);
	
	self.PlaceHolderBoxes = {};
	self.PlaceHolderBoxes[1] = self:CreatePlaceHolderBox(self.PlaceHolderLabel:GetLeft() + self.PlaceHolderLabel:GetWidth());
	self.PlaceHolderBoxes[2] = self:CreatePlaceHolderBox(self.PlaceHolderBoxes[1]:GetLeft() + self.PlaceHolderBoxes[1]:GetWidth() + Constants.Padding.Columns);

	self.TranslationBox = CreateTextBox(self, true, true);
	self.TranslationBox:SetLeft(left);
	self.TranslationBox:SetHeight(Constants.Sizes.PhraseBox);
	self.TranslationBox:SetReadOnly(true);
	
	self.Shortcut = Turbine.UI.Lotro.Shortcut();
	self.Shortcut:SetType(Turbine.UI.Lotro.ShortcutType.Alias);
	self.Shortcut:SetData("/");
	
	self.Quickslot = Turbine.UI.Lotro.Quickslot();
	self.Quickslot:SetParent(self);
	self.Quickslot:SetSize(Constants.Sizes.ReadButton - 2, Constants.Sizes.ReadButton - 2);
	self.Quickslot:GetAllowDrop(false);
	self.Quickslot:SetVisible(false);
	self:SetShortcut();
	
	-- Fix for dragging short-cut
	self.Quickslot.DragDrop = function(sender, args)
		self:SetShortcut(); -- restore shortcut
	end
	
	self.QuickslotIcon = Turbine.UI.Control();
	self.QuickslotIcon:SetParent(self);
	self.QuickslotIcon:SetSize(Constants.Sizes.ReadButton, Constants.Sizes.ReadButton);
	self.QuickslotIcon:SetZOrder(self.Quickslot:GetZOrder() + 1);
	self.QuickslotIcon:SetMouseVisible(false);
	self.QuickslotIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	--self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Normal);
	SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Normal");
	self.QuickslotIcon.EnabledChanged = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			--self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Normal);
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Normal");
		else
			--self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Inactive);
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Inactive");
		end
	end
	self.QuickslotIcon:SetEnabled(false);
	
	self.HideTranslationCheckBox = CreateCheckBox(self, Strings["hide_translation"], left, 0);
	self.HideTranslationCheckBox:SetSize(Globals.Sizes.CheckBox.Width, Globals.Sizes.CheckBox.Height);
	self.HideTranslationCheckBox:SetChecked(Settings.NeoSindarinWindow.HideTranslation or false);
	self.HideTranslationCheckBox.CheckedChanged = function (sender, args)
		self:SetShortcutText();
	end
	
	self.Quickslot.MouseEnter = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			--self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Hover);
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Hover");
		end
	end
	self.Quickslot.MouseLeave = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			--self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Normal);
			SetIconImage(shortcutImages, self.QuickslotIcon, "MainShortcut_Normal");
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self.UIHidden then
			if self:IsVisible() then
				Settings.NeoSindarinWindow.Visible = "yes";
			else
				Settings.NeoSindarinWindow.Visible = "no";
				self:SaveSettings(false);
			end
		end
		
		if self:IsVisible() then
			CheckWindowPosition(self);
		end
	end

	self:LayoutControls();
	self:PopulatePhrases();
end

function NeoSindarinWindow:CreatePlaceHolderBox(left)
	local placeHolderBox = CreateTextBox(self, false, true);
	placeHolderBox:SetWantsUpdates(false);
	placeHolderBox:SetEnabled(false);
	placeHolderBox:SetForeColor(Globals.Colours.Disabled);
	placeHolderBox:SetLeft(left);
	placeHolderBox.PlaceHolderText = "";
	placeHolderBox.FocusGained = function(sender, args)
		sender:SetWantsUpdates(true);
	end
	placeHolderBox.FocusLost = function(sender, args)
		sender:SetWantsUpdates(false);
		self:ShowTranslatedText(true);
	end
	placeHolderBox.Update = function(sender, args)
		local newText = sender:GetText();
		if sender.PlaceHolderText ~= newText then
			sender.PlaceHolderText = newText;
			self:ShowTranslatedText(true);
		end
	end
	
	return placeHolderBox;
end

function NeoSindarinWindow:LayoutControls()
	self.SortIcon:SetLeft(self:GetWidth() - Constants.Padding.Right - Constants.Sizes.SortIcon.Width);
	self.SortLabel:SetLeft(self:GetWidth() - Constants.Padding.Right - Constants.Sizes.SortIcon.Width - Constants.Sizes.SortLabel - Constants.Padding.Lines);
	
	self.DialectInfoLabel:SetWidth(self:GetWidth() - self.DialectInfoLabel:GetLeft() - Constants.Padding.Right);

	self.MoreInfoButton:SetLeft(self:GetWidth() - Constants.Padding.Right - self.MoreInfoButton:GetWidth());
	self.QuickslotIcon:SetPosition((self:GetWidth() - self.QuickslotIcon:GetWidth()) / 2, self:GetHeight() - Constants.Padding.Bottom - self.QuickslotIcon:GetHeight());
	self.Quickslot:SetPosition(self.QuickslotIcon:GetLeft() + 1, self.QuickslotIcon:GetTop() + 1);
	
	self.HideTranslationCheckBox:SetTop(self.QuickslotIcon:GetTop());

	local width = self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right;
	
	self.TranslationBox:SetTop(self.Quickslot:GetTop() - Constants.Padding.Lines - self.TranslationBox:GetHeight());
	self.TranslationBox:SetWidth(width);

	self.PlaceHolderLabel:SetTop(self.TranslationBox:GetTop() - Constants.Padding.Lines - self.PlaceHolderLabel:GetHeight());
	self.PlaceHolderBoxes[1]:SetTop(self.TranslationBox:GetTop() - Constants.Padding.Lines - self.PlaceHolderBoxes[1]:GetHeight());
	self.PlaceHolderBoxes[2]:SetTop(self.PlaceHolderBoxes[1]:GetTop());
	self.MoreInfoButton:SetTop(self.PlaceHolderBoxes[1]:GetTop());
	
	local adjustHeight = 0;

	if self.InfoBox:IsVisible() then
		adjustHeight = self.InfoBox:GetHeight() + Constants.Padding.Lines;
		self.InfoBox:SetWidth(width);
		self.InfoBox:SetTop(self.PlaceHolderBoxes[1]:GetTop() - adjustHeight);
	end
	
	self.PhrasesListBox:SetSize(width, self.PlaceHolderBoxes[1]:GetTop() - self.PhrasesListBox:GetTop() - Constants.Padding.Lines - adjustHeight);
end

function NeoSindarinWindow:Show()
	self:SetVisible(true);
	self:Activate();
end

function NeoSindarinWindow:Hide()
	if self:IsVisible() then
		self:SetVisible(false);
	end
end

function NeoSindarinWindow:Toggle()
	if self:IsVisible() then
		self:Hide();
	else
		self:Show();
	end
end

function NeoSindarinWindow:SaveSettings(showMessage)
	Settings.NeoSindarinWindow.Position.Top = tostring(self:GetTop());
	Settings.NeoSindarinWindow.Position.Left = tostring(self:GetLeft());
	Settings.NeoSindarinWindow.Size.Width = tostring(self:GetWidth());
	Settings.NeoSindarinWindow.Size.Height = tostring(self:GetHeight());
	Settings.NeoSindarinWindow.DefaultDialect = self.DialectList:GetSelectedValue();
	Settings.NeoSindarinWindow.DefaultFormality = self.FormalityList:GetSelectedValue();
	Settings.NeoSindarinWindow.DefaultChannel = self.ChannelList:GetSelectedValue();
	Settings.NeoSindarinWindow.HideTranslation = self.HideTranslationCheckBox:IsChecked();
	
	Settings.ToggleWindow.Position.Left = tostring(toggleWindow:GetLeft());
	Settings.ToggleWindow.Position.Top = tostring(toggleWindow:GetTop());

	NeoSindarinSave(Turbine.DataScope.Account, "NeoSindarinSettings", Settings);
end

function NeoSindarinWindow:PopulatePhrases()
	self.PhrasesListBox:ClearItems();
	self.DisplayedPhrases = {};
	
	local categoryKey = self.SelectedCategory;
	local keyword = self.FilterBox.SaveText or "";
	
	if categoryKey ~= nil then
		for phraseIndex, phraseData in ipairs(NeoSindarinDB.Phrases) do 
			if categoryKey == "" or phraseData.CategoryKey == categoryKey then
				table.insert(self.DisplayedPhrases, phraseData);
			end
		end
		
		self:SortPhrases();
	end
end

function NeoSindarinWindow:SortPhrases()
	if self.SortOrder ~= nil then
		table.sort(self.DisplayedPhrases, function(phraseAData, phraseBData)
			local phraseA = (phraseAData[LANGUAGE] or phraseAData["EN"]):gsub("___%s*", ""):gsub("^(.*)(%([^%)]*%)%s*)(.*)$", "%1%3%2");
			local phraseB = (phraseBData[LANGUAGE] or phraseBData["EN"]):gsub("___%s*", ""):gsub("^(.*)(%([^%)]*%)%s*)(.*)$", "%1%3%2");
			
			if self.SortOrder == "Asc" then
				return phraseA < phraseB;
			else
				return phraseA > phraseB;
			end
		end)
	end
	
	local keyword = self.FilterBox.SaveText or "";
	self:FilterPhrases(keyword, false);
end

function NeoSindarinWindow:FilterPhrases(keyword, reselectLine)
	if reselectLine == nil then
		reselectLine = true;
	end
	
	local currentPhrase = self.SelectedPhrase
	self.PhrasesListBox:ClearItems();
	
	local newIndex = 1;
	local reselectPhraseIndex = 0;
	for index, phraseData in ipairs(self.DisplayedPhrases) do 
		local phraseText = phraseData[LANGUAGE] or phraseData["EN"];
		local shortInfo = phraseData["ShortInfo"];
		if shortInfo ~= nil then
			phraseText = phraseText .. " (" .. (shortInfo[LANGUAGE] or shortInfo["EN"]) .. ")";
		end
		if string.len(keyword) == 0 or phraseText:Contains(keyword) then
			local disabledKey = nil;
			if phraseData["Woodelven_All"] == "" then
				disabledKey = "Woodelven";
			elseif phraseData["Exilic_All"] == "" then
				disabledKey = "Exilic";
			end
			self.PhrasesListBox:AddItem(phraseText, phraseData, disabledKey);
			if currentPhrase ~= nil and phraseData == currentPhrase then
				reselectPhraseIndex = newIndex;
			end
			newIndex = newIndex + 1;
		end
	end
	
	if reselectLine and reselectPhraseIndex > 0 then
		self.PhrasesListBox:SelectLine(reselectPhraseIndex);
	--else
	--	self:ShowTranslatedText();
	end
end

function NeoSindarinWindow:ShowTranslatedText(doFocus)
	local neoSindarinText = nil;
	local phraseText = nil;
	local enableButton = false;
	local neoSindarinSplits = 0;
	local phraseSplits = 0;
	local placeholdersVisible = 0;
	
	if self.LastText == nil then
		self.LastText = "";
	end
	
	if self.SelectedPhrase ~= nil and self.SelectedDialect ~= nil and self.SelectedFormality ~= nil then
		local phrase = self.SelectedPhrase;
		
		neoSindarinText = phrase[self.SelectedDialect .. "_" .. self.SelectedFormality] or phrase[self.SelectedDialect .. "_All"] or phrase["All"];
		neoSindarinText = neoSindarinText:gsub("____*", "___");
		phraseText = (phrase[LANGUAGE] or phrase["EN"]);
		phraseText = phraseText:gsub("____*", "___");
		enableButton = true;

		if neoSindarinText == "" then
			self.TranslationBox:SetText(Strings["no_translation"]);
			enableButton = false;
		else
			neoSindarinSplits = neoSindarinText:Split("___", true);
			phraseSplits = phraseText:Split("___", true);
			
			placeholdersVisible = table.getn(neoSindarinSplits) - 1;
			if placeholdersVisible > 0 then
				neoSindarinText = neoSindarinSplits[1];
				phraseText = phraseSplits[1];
				for index = 1, 2 do
					if placeholdersVisible > index - 1 then
						if self.PlaceHolderBoxes[index].PlaceHolderText ~= "" then
							neoSindarinText = neoSindarinText .. self.PlaceHolderBoxes[index].PlaceHolderText;
							phraseText = phraseText .. self.PlaceHolderBoxes[index].PlaceHolderText;
						else
							neoSindarinText = neoSindarinText .. "___";
							phraseText = phraseText .. "___";
							enableButton = false;
						end
						neoSindarinText = neoSindarinText .. neoSindarinSplits[index + 1];
						phraseText = phraseText .. phraseSplits[index + 1];
					end
				end
			end
			self.TranslationBox:SetText(neoSindarinText);
		end
	else
		self.TranslationBox:SetText("");
	end
		
	if placeholdersVisible > 0 then
		self.PlaceHolderLabel:SetForeColor(Globals.Colours.Label);
		self.PlaceHolderBoxes[1]:SetForeColor(Globals.Colours.TextBox);
		if placeholdersVisible > 1 then
			self.PlaceHolderBoxes[2]:SetForeColor(Globals.Colours.TextBox);
		else
			self.PlaceHolderBoxes[2]:SetEnabled(false);
			self.PlaceHolderBoxes[2]:SetForeColor(Globals.Colours.Disabled);
		end
		if self.LastText ~= self.SelectedPhraseText or not self.PlaceHolderBoxes[1]:IsEnabled() or (placeholdersVisible > 1 and not self.PlaceHolderBoxes[2]:IsEnabled()) then
			self.PlaceHolderBoxes[1]:SetEnabled(true);
			if placeholdersVisible > 1 then
				self.PlaceHolderBoxes[2]:SetEnabled(true);
			end
			if doFocus then
				self.PlaceHolderBoxes[1]:Focus();
			end
		end
	else
		self.PlaceHolderBoxes[1]:SetEnabled(false);
		self.PlaceHolderBoxes[2]:SetEnabled(false);
		self.PlaceHolderLabel:SetForeColor(Globals.Colours.Disabled);
		self.PlaceHolderBoxes[1]:SetForeColor(Globals.Colours.Disabled);
		self.PlaceHolderBoxes[2]:SetForeColor(Globals.Colours.Disabled);
	end
	
	self.LastText = self.SelectedPhraseText;
	
	if enableButton then
		self.NeoSindarinText = neoSindarinText;
		self.PhraseText = phraseText;
		self.Quickslot:SetVisible(true);
		self.QuickslotIcon:SetEnabled(true);
		self:SetShortcutText();
	else
		self.Quickslot:SetVisible(false);
		self.QuickslotIcon:SetEnabled(false);
	end
end

function NeoSindarinWindow:SetShortcutText()
	if self.Quickslot:IsVisible() then
		if self.HideTranslationCheckBox:IsChecked() then
			self:SetShortcut(self.NeoSindarinText);
		else
			self:SetShortcut(self.NeoSindarinText .. " ((" .. self.PhraseText .. "))");
		end
	end
end

function NeoSindarinWindow:SetShortcut(text)
	if text ~= nil then
		self.Shortcut:SetData(self.ChannelList:GetSelectedValue() .. " " .. text);
	end
	self.Quickslot:SetShortcut(self.Shortcut);
end